/* 
 *
 *
 * Copyright (C) 2010 Michal Malek <michalm@jabster.pl>
 *
 * This file is part of the K3b project.
 * Copyright (C) 1998-2008 Sebastian Trueg <trueg@k3b.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#ifndef _K3B_OGG_VORBIS_ENCODER_DEFAULTS_H_
#define _K3B_OGG_VORBIS_ENCODER_DEFAULTS_H_

const bool DEFAULT_MANUAL_BITRATE = false;
const int DEFAULT_QUALITY_LEVEL = 4;
const int DEFAULT_BITRATE_UPPER = -1;
const int DEFAULT_BITRATE_NOMINAL = -1;
const int DEFAULT_BITRATE_LOWER = -1;
const int DEFAULT_SAMPLERATE = 44100;

#endif // _K3B_OGG_VORBIS_ENCODER_DEFAULTS_H_
